@@ -217,6 +217,20 @@ Zarafa.plugins.filepreviewer.ViewerPanel = Ext.extend(Ext.Panel, {
 			this.defaultScale = 0.8;
 		}
 
+		var isDeskApp = Ext.isFunction(Zarafa.isDeskApp) ? Zarafa.isDeskApp() : Ext.isDefined(window.nw);
+		if (isDeskApp) {
+			// Render a webview for DeskApp, to make it more secure
+			// (it also solves issues with namespace collisions)
+			this.tag = 'webview';
+			// First load a blank page to be able to inject the cookies with credentials
+			// Store the src of the viewer so we can redirect to it after the cookies have
+			// been injected
+			var path = container.getSettingsModel().get('zarafa/v1/plugins/filepreviewer/path');
+			this.viewerSrc = this.src;
+			this.src = location.href + path + '/html/blank.html';
+		} else {
+			this.tag = 'iframe';
+		}
 		Ext.applyIf(config, {
 			xtype : 'filepreviewer.viewerpanel',
 			layout: 'anchor',
@@ -225,7 +239,7 @@ Zarafa.plugins.filepreviewer.ViewerPanel = Ext.extend(Ext.Panel, {
 				xtype : 'component',
 				ref : 'iframe',
 				autoEl: {
-					tag            : 'iframe',
+					tag            : this.tag,
 					src            : this.src,
 					style          : {
 						width : '100%',
@@ -235,7 +249,8 @@ Zarafa.plugins.filepreviewer.ViewerPanel = Ext.extend(Ext.Panel, {
 					allowfullscreen: true
 				},
 				listeners : {
-					afterrender : this.onAfterRender
+					afterrender : this.onAfterRender,
+					scope: this
 				}
 			}],
 			title : Ext.util.Format.htmlEncode(this.title),
@@ -342,21 +357,43 @@ Zarafa.plugins.filepreviewer.ViewerPanel = Ext.extend(Ext.Panel, {
 	 * Event handler which is fired after the {@link Zarafa.plugins.filepreviewer.ViewerPanel Panel}
 	 * has been {@link Ext.Component#afterrender rendered}.
 	 * This will register the load event handler on {@link Ext.Component IFrame}.
+	 * @param {Ext.Component} iframe The component that encapsulates the iframe
 	 */
-	onAfterRender : function ()
+	onAfterRender : function (iframe)
 	{
-		var self = this.refOwner;
-		this.getEl().on('load', self.onLoadFrame, this.getEl(), {single : true});
+		// Add a load event handler for the iframe
+		this.getEl().on('load', this.onLoadFrame, this.getEl(), {single : true});
+		// Add a load event handler for the webview
+		this.getEl().on('contentload', this.onLoadWebview, this, {single : true});
+
+		// Add some event listeners when we are using webviews (i.e. in DeskApp)
+		if ( this.tag === 'webview' ){
+			iframe.el.dom.addEventListener('permissionrequest', this.handlePermissionRequests);
+			iframe.el.dom.addEventListener('dialog', this.handleDialogRequests, this);
+		}
 	},
 
 	/**
 	 * Handler for the 'load' event of iframe, fired immediately after iframe is loaded.
+	 * Passes the keydown event for the Escape key to the parent window, so it can close
+	 * the preview window.
 	 * Get the {@link Ext.Button download button} of iframe body and
 	 * it will skip showing the requester on {@link Ext.Button download button} click event
 	 */
 	onLoadFrame : function ()
 	{
 		var frameDom = this.dom.contentDocument;
+
+		var self = this;
+		frameDom.addEventListener('keydown', function(origEvent) {
+			// Pass the esc keydown event to the parent window, so
+			// it can close the preview window
+			if ( origEvent.keyCode === 27 ) {
+				var event = new KeyboardEvent(origEvent.type, origEvent);
+				self.dom.dispatchEvent(event);
+			}
+		});
+
 		var download = frameDom.getElementById('download');
 		download = Ext.get(download);
 
@@ -366,6 +403,102 @@ Zarafa.plugins.filepreviewer.ViewerPanel = Ext.extend(Ext.Panel, {
 		download.on('click', function () {
 			Zarafa.core.Util.skipRequester = true;
 		});
+	},
+
+	/**
+	 * Handler for the contentload event of the webview. Will inject the WebApp cookies
+	 * into the webview, so documents can be loaded
+	 */
+	onLoadWebview : function()
+	{
+		var iframeCookieStoreId = this.iframe.el.dom.getCookieStoreId();
+		var cookieCounter = 0;
+		chrome.cookies.getAll({url: location.href}, function(cookies) {
+			cookies.forEach( function(c) {
+				var cookie = {
+					url: location.href,
+					name: c.name,
+					value: c.value,
+					path: c.path,
+					secure: c.secure,
+					httpOnly: c.httpOnly,
+					sameSite: c.sameSite,
+					storeId: iframeCookieStoreId
+				};
+				chrome.cookies.set(cookie, function() {
+					if ( ++cookieCounter === cookies.length ) {
+						this.iframe.el.dom.src = this.viewerSrc;
+					}
+				}.bind(this));
+			}.bind(this));
+		}.bind(this));
+	},
+
+	/**
+	 * Handler for the dialog event of WEBVIEW elements. Will handle alert, prompt,
+	 * and confirm dialogs
+	 * @param  {Event} e The dialog event
+	 */
+	handleDialogRequests : function(e)
+	{
+		// Handle alerts
+		if ( e.messageType === 'alert' ) {
+			window.alert(e.messageText);
+		}
+
+		// Handle confirm dialogs
+		else if ( e.messageType === 'confirm' ) {
+			var confirmation =  window.confirm(e.messageText);
+
+			if ( confirmation ) {
+				e.dialog.ok();
+			} else {
+				e.dialog.cancel();
+			}
+		}
+
+		// Handle prompts
+		else if ( e.messageType === 'prompt' ){
+			var wprompt = window.prompt(e.messageText);
+
+			if ( wprompt === null ){
+				e.dialog.cancel();
+			} else {
+				e.dialog.ok(wprompt);
+			}
+		}
+
+	},
+
+	/**
+	 * Handler for the permissionrequest event of WEBVIEW elements. Will handle the request
+	 * by its type.
+	 * Possible types are media, geolocation, pointerLock, download, loadplugin and fullscreen.
+	 * For now we deny geolocation, fullscreen and pointerLock requests.
+	 * @param {Event} e The permissionrequest event
+	 */
+	handlePermissionRequests : function(e)
+	{
+		e.preventDefault();
+		switch (e.permission) {
+			// Allow
+			case 'download':
+			case 'media':
+			case 'loadplugin':
+			case 'fullscreen':
+				e.request.allow();
+			break;
+			// Deny
+			case 'pointerLock':
+			case 'geolocation':
+				e.request.deny();
+			break;
+			// also deny all other, not yet known, requests
+			default:
+				e.request.deny();
+			break;
+
+		}
 	}
 });
 
