@@ -8,9 +8,9 @@
  */
 function stringToUTF8Encode($str)
 {
-	if (!mb_detect_encoding($str, "UTF-8", true)) {
-		$str = utf8_encode($str);
-	}
+//	if (!mb_detect_encoding($str, "UTF-8", true)) {
+//		$str = utf8_encode($str);
+//	}
 	return $str;
 }
 
