@@ -486,7 +486,13 @@ class Backend extends AbstractBackend implements iFeatureQuota, iFeatureVersionI
 
 		if ($info && $info->isDirectory()) {
 			try {
-				$info = $this->smb_share->rmdir($path);
+				$content = $this->smb_share->dir($path);
+				// If folder is not empty then delete all content of folder.
+				if (empty($content)) {
+					$info = $this->smb_share->rmdir($path);
+				} else {
+					$info = $this->deleteAllContent($path);
+				}
 			} catch (NotFoundException $e) {
 				$this->log("RMDIR NFE: " . print_r($e, true));
 				throw new BackendException($this->parseErrorCodeToMessage(self::SMB_ERR_NOTFOUND), self::SMB_ERR_NOTFOUND);
@@ -521,6 +527,22 @@ class Backend extends AbstractBackend implements iFeatureQuota, iFeatureVersionI
 		return true;
 	}
 
+	/**
+	 * Delete all content of directory
+	 *
+	 * @param string $path file/directory path
+	 */
+	function deleteAllContent($path)
+	{
+		$content = $this->ls($path);
+		// Delete all contents of folder.
+		foreach ($content as $key => $record) {
+			$this->delete($key);
+		}
+		// Finally delete the folder.
+		$this->delete($path);
+	}
+
 	/**
 	 * Move a file or collection on webdav server (serverside)
 	 * If you set param overwrite as true, the target will be overwritten.
