@@ -37,6 +37,11 @@
 		 */
 		private $store = false;
 
+		/**
+		 * MAPI Message Store objects.
+		 */
+		private $stores = [];
+
 		/**
 		 * The PR_MAPPING_SIGNATURE for the current store
 		 */
@@ -54,8 +59,6 @@
 		 */
 		private $mapping = array();
 
-		function __construct(){}
-
 		/**
 		 * Initialize the class by opening the default message store. This is done only once.
 		 */
@@ -63,20 +66,20 @@
 		{
 			if ($this->init) {
 				return;
-			} else {
-				$this->store = $GLOBALS["mapisession"]->getDefaultMessageStore();
-				if (!$this->storeMapping) {
-					$this->storeMapping = mapi_getprops($this->store, array(PR_MAPPING_SIGNATURE));
-					$this->storeMapping = isset($this->storeMapping[PR_MAPPING_SIGNATURE]) ? bin2hex($this->storeMapping[PR_MAPPING_SIGNATURE]) : '0';
-
-					// Ensure the mapping exists
-					if (!isset($this->mapping[$this->storeMapping])) {
-						$this->mapping[$this->storeMapping] = array();
-					}
-				}
-
-				$this->init = true;
 			}
+			
+			$this->store = $this->getStore();
+			$storeMapping = $this->getStoreMappingSignature($this->store);
+			
+			if ($this->storeMapping !== $storeMapping) {
+				$this->storeMapping = $storeMapping;
+				// Ensure the mapping exists
+				if (!isset($this->mapping[$this->storeMapping])) {
+					$this->mapping[$this->storeMapping] = array();
+				}
+			}
+
+			$this->init = true;
 		}
 
 		/**
@@ -91,6 +94,72 @@
 		{
 			$this->init = false;
 			$this->store = false;
+			$this->stores = [];
+		}
+
+		/**
+		 * Setter function which set the store.
+		 *
+		 * @param object|array|boolean MAPI Message Store Object or array of MAPI Message Store Objects, false if storeid is not found in the request.
+		 */
+		function setStore($store = false)
+		{
+			$stores = [];
+			if ($store === false) {
+				$this->stores = $stores;
+				return;
+			}
+
+			if (is_array($store)) {
+				$stores = $store;
+			} else {
+				$stores = array($store);
+				$this->store = $store;
+			}
+
+			foreach($stores as $key => $store) {
+				$storeMapping = $this->getStoreMappingSignature($store);
+				if ($this->storeMapping !== $storeMapping) {
+					$this->stores[$storeMapping] = $store;
+
+				}
+			}
+		}
+
+		/**
+		 * Getter function which get the store.
+		 *
+		 * @return object MAPI Message Store Object
+		 */
+		function getStore()
+		{
+			return $this->store !== false ? $this->store : $GLOBALS["mapisession"]->getDefaultMessageStore();
+		}
+
+		/**
+		 * Function which used to get the PR_MAPPING_SIGNATURE value from given store.
+		 *
+		 * @param object MAPI Message Store Object
+		 * @return string PR_MAPPING_SIGNATURE of the given MAPI Message Store if exists else 0.
+		 */
+		private function getStoreMappingSignature($store)
+		{
+			$storeMapping = mapi_getprops($store, array(PR_MAPPING_SIGNATURE));
+			return isset($storeMapping[PR_MAPPING_SIGNATURE]) ? bin2hex($storeMapping[PR_MAPPING_SIGNATURE]) : '0';
+		}
+
+		/**
+		 * Helper function which set the store as a active store and storeMapping.
+		 *
+		 * @param object MAPI Message Store Object
+		 */
+		function setActiveStore($store)
+		{
+			$storeMapping = $this->getStoreMappingSignature($store);
+			if ($this->storeMapping !== $storeMapping) {
+				$this->store = $store;
+				$this->storeMapping = $storeMapping;
+			}
 		}
 
 		/**
@@ -129,6 +198,28 @@
 			return $this->mapping[$this->storeMapping]['recipient'];
 		}
 
+		/**
+		 * Returns the properties for Out of office settings in a message
+		 * @return array properties for Out of office settings.
+		 */
+		function getOutOfOfficeProperties()
+		{
+			$this->Init();
+
+			if (!isset($this->mapping[$this->storeMapping]['oofsettings'])) {
+				$properties["set"] = PR_EC_OUTOFOFFICE;
+				$properties["entryid"] = PR_MAILBOX_OWNER_ENTRYID;
+				$properties["store_entryid"] = PR_ENTRYID;
+				$properties["message"] = PR_EC_OUTOFOFFICE_MSG;
+				$properties["subject"] = PR_EC_OUTOFOFFICE_SUBJECT;
+				$properties["from"] = PR_EC_OUTOFOFFICE_FROM;
+				$properties["until"] = PR_EC_OUTOFOFFICE_UNTIL;
+				$this->mapping[$this->storeMapping]['oofsettings'] = getPropIdsFromStrings($this->store, $properties);
+			}
+
+			return $this->mapping[$this->storeMapping]['oofsettings'];
+		}
+
 		/**
 		 * Returns the properties for a meeting request
 		 */
@@ -276,6 +367,8 @@
 				$properties["reply_time"] = "PT_SYSTIME:PSETID_Appointment:0x8220";
 				$properties["reply_name"] = "PT_STRING8:PSETID_Appointment:0x8230";
 
+				$properties["onlinemeetingurl"] = "PT_STRING8:PS_PUBLIC_STRINGS:OnlineMeetingExternalLink";
+
 				$this->mapping[$this->storeMapping]['appointment'] = getPropIdsFromStrings($this->store, $properties);
 			}
 
@@ -689,6 +782,7 @@
 				$properties["display_type_ex"] = PR_DISPLAY_TYPE_EX;
 				$properties["account"] = PR_ACCOUNT;
 				$properties["address_type"] = PR_ADDRTYPE;
+				$properties["title"] = PR_TITLE;
 				$properties["object_type"] = PR_OBJECT_TYPE;
 				$properties["search_key"] = PR_SEARCH_KEY;
 				$properties["office_telephone_number"] = PR_OFFICE_TELEPHONE_NUMBER;
@@ -933,6 +1027,9 @@
 				// Archiver property
 				$properties["stubbed"] = "PT_BOOLEAN:PSETID_Archive:stubbed";
 
+				// Conversation properties
+				$properties["conversation_topic"] = PR_CONVERSATION_TOPIC;
+
 				// Allowing to hook in and add more properties
 				$GLOBALS['PluginManager']->triggerHook("server.core.properties.mailproperties", array(
 					'properties' =>& $properties
@@ -1257,7 +1354,10 @@
 				$properties["rule_provider_data"] = PR_RULE_PROVIDER_DATA;
 				$properties["rule_level"] = PR_RULE_LEVEL;
 				$properties["rule_user_flags"] = PR_RULE_USER_FLAGS;
-
+				$properties["rule_msg_atleast_size_unit"] = PR_RULE_ATLEAST_MESSAGE_SIZEUNIT;
+				$properties["rule_msg_atmost_size_unit"] = PR_RULE_ATMOST_MESSAGE_SIZEUNIT;
+				$properties["rule_exception_atleast_size_unit"] = PR_RULE_EXCEPTION_ATLEAST_MESSAGE_SIZEUNIT;
+				$properties["rule_exception_atmost_size_unit"] = PR_RULE_EXCEPTION_ATMOST_MESSAGE_SIZEUNIT;
 				$this->mapping[$this->storeMapping]['rules'] = $properties;
 			}
 
