@@ -47,14 +47,13 @@
 
 						// If userStore not found it means user is not exists in address book
 						if($userStore === false) {
-							$msg = "The contact \"%s\" could not be displayed because it could not be retrieved or has been deleted (e.g. kopano-contact deleted from LDAP)";
+							$msg = "The contact \"%s\" could not be displayed because it could not be retrieved or has been deleted (e.g. kopano-contact deleted from LDAP or AD)";
 
 							error_log(sprintf($msg, $action["message_action"]["username"]));
 
 							$e->setTitle(_('Contact not found'));
-							$e->setDisplayMessage(_("Contact information could not be displayed because the server 
-							had trouble retrieving the information. Please contact your system administrator if the
-							problem persists."));
+							$e->setDisplayMessage(_("Contact information could not be displayed because the server had trouble retrieving the information.") .
+							_("Please contact your system administrator if the problem persists."));
 							throw $e;
 							return false;
 						}
