@@ -135,6 +135,8 @@
 			parent::handleException($e, $actionType, $store, $parententryid, $entryid, $action);
 		}
 
+		function updateMessageListData(&$data, $store, $folderEntryid, $action, $actionType){}
+
 		/**
 		 * Function which retrieves a list of messages in a folder
 		 * @param object $store MAPI Message Store Object
@@ -157,6 +159,8 @@
 				$limit = false;
 				if(isset($action['restriction']['limit'])){
 					$limit = $action['restriction']['limit'];
+				} else {
+					$limit = $GLOBALS['settings']->get('zarafa/v1/main/page_size', 50);
 				}
 
 				$isSearchFolder = isset($action['search_folder_entryid']);
@@ -165,7 +169,6 @@
 				// Get the table and merge the arrays
 				$data = $GLOBALS["operations"]->getTable($store, $entryid, $this->properties, $this->sort, $this->start, $limit, $this->restriction);
 
-
 				// If the request come from search folder then no need to send folder information
 				if (!$isSearchFolder) {
 					// Open the folder.
@@ -191,12 +194,13 @@
 					'store' => $store,
 					'entryid' => $entryid,
 					'action' => $action,
-					'data' =>& $data
+					'data' =>& $data,
 				));
 
 				// unset will remove the value but will not regenerate array keys, so we need to
 				// do it here
 				$data["item"] = array_values($data["item"]);
+
 				$this->addActionData($actionType, $data);
 				$GLOBALS["bus"]->addData($this->getResponseData());
 			}
